### R code from vignette source 'BrCaMS'
### Encoding: UTF-8

###################################################
### code chunk number 1: BrCaMS.rnw:4-9
###################################################
options( width=90,
#        prompt=" ", continue=" ", # Absence of prompts makes it easier for
                                   # students to copy from the final pdf document
         SweaveHooks=list( fig=function()
         par(mar=c(3,3,1,1),mgp=c(3,1,0)/1.6,las=1,bty="n") ) )


###################################################
### code chunk number 2: read
###################################################
library( Epi )
library( popEpi )
load( file="./BrCa.rda" )
# data( BrCa )
head( BrCa )


###################################################
### code chunk number 3: defLex
###################################################
set.seed( 1952 )
Lbc <- Lexis( entry = list( tfd = 0,
                              A = age  + runif(nrow(BrCa)),
                              P = year + runif(nrow(BrCa)) ),
               exit = list( tfd = tox ),
        exit.status = xst,
                 id = pid,
               data = BrCa )
summary( Lbc )
names( Lbc )


###################################################
### code chunk number 4: cut
###################################################
Rbc <- cutLexis( Lbc, 
                 cut = pmin( Lbc$tor, Lbc$tom, na.rm=TRUE ),
           timescale = "tfd",
    precursor.states = "Alive",
           new.state = "Rel",
        split.states = TRUE,
           new.scale = "tfr" )
summary( Rbc, timeScale = TRUE )


###################################################
### code chunk number 5: box-r
###################################################
boxes( Rbc, boxpos=list(x=c(15,15,85,85),
                        y=c(85,15,85,15)),
            show.BE=TRUE, scale.R=100, )


###################################################
### code chunk number 6: BrCaMS.rnw:113-116
###################################################
system.time(
Sbc <- splitLexis( Rbc, breaks=seq(0,100,1/12), "tfd" ) )
summary( Sbc )


###################################################
### code chunk number 7: BrCaMS.rnw:120-123
###################################################
system.time(
Sbc <- splitMulti( Rbc, tfd=seq(0,100,1/12) ) )
summary( Sbc )


###################################################
### code chunk number 8: BrCaMS.rnw:131-136
###################################################
Stbc <- stack( Sbc )
round( ftable( xtabs( cbind(lex.Fail,lex.dur) ~ lex.Tr + lex.Xst, 
                      data=Stbc ),
               row.vars=c(3,1),
       1 ) )


###################################################
### code chunk number 9: BrCaMS.rnw:169-179
###################################################
( kd.ad <- with( subset( Sbc, lex.Cst=="Alive" & lex.Xst=="Dead"),
                 quantile( tfd+lex.dur, probs=(1:4-0.5)/4) ) )
( kd.ar <- with( subset( Sbc, lex.Cst=="Alive" & lex.Xst=="Rel"),
                 quantile( tfd+lex.dur, probs=(1:4-0.5)/4) ) )
( kd.rd <- with( subset( Sbc, lex.Cst=="Rel" & lex.Xst=="Dead(Rel)"),
                 quantile( tfd+lex.dur, probs=(1:4-0.5)/4) ) )
( kr.rd <- with( subset( Sbc, lex.Cst=="Rel" & lex.Xst=="Dead(Rel)"),
                 quantile( tfr+lex.dur, probs=(1:4-0.5)/4) ) )
( ka.rd <- with( subset( Sbc, lex.Cst=="Rel" & lex.Xst=="Dead(Rel)"),
                 quantile( tfd-tfr, probs=(1:4-0.5)/4) ) )


###################################################
### code chunk number 10: BrCaMS.rnw:186-201
###################################################
m.ad <- glm( (lex.Xst=="Dead") ~ Ns( tfd, knots=kd.ad ),
             offset = log( lex.dur ),
             family = poisson,
               data = subset( Sbc, lex.Cst=="Alive" ) )
m.ar <- glm( (lex.Xst=="Rel") ~ Ns( tfd, knots=kd.ar ),
             offset = log( lex.dur ),
             family = poisson,
               data = subset( Sbc, lex.Cst=="Alive" ) )
m.rd <- glm( (lex.Xst=="Dead(Rel)") ~ Ns( tfd, knots=kd.rd ),
             offset = log( lex.dur ),
             family = poisson,
               data = subset( Sbc, lex.Cst=="Rel" ) )
x.rd <- update( m.rd, . ~ . + Ns( tfr, knots=kr.rd) )
r.rd <- update( x.rd, . ~ . - Ns( tfd, knots=kd.rd) )
anova( m.rd, x.rd, r.rd, test="Chisq" )


###################################################
### code chunk number 11: rate-pr
###################################################
nd <- data.frame( tfd = seq(0,15,0.1),
              lex.dur = 100 )
ad.rate <- ci.pred( m.ad, nd )
ar.rate <- ci.pred( m.ar, nd )
rd.rate <- ci.pred( m.rd, nd )


###################################################
### code chunk number 12: pr-pl
###################################################
clr <- rainbow(3) # ; yl <- c(0.03,60)
matplot( nd$tfd, cbind( ad.rate,
                        ar.rate,
                        rd.rate ),
         type="l", lty=1, lwd=c(3,1,1), col=rep(clr,each=3), las=1,
         log="y", xlab="Time since diagnosis (years)",
                  ylab="Rate per 100 PY" )
text( par("usr")[2]*0.95, (10^par("usr"))[3]*1.4^(1:3), 
      c("A->D","A->R","R->D"), col=clr, adj=1, font=2 )
matlines( nd$tfd, ci.ratio( rd.rate, ad.rate ), 
          lty=1, lwd=c(3,1,1), col=gray(0.6) )
abline( h=1, col=gray(0.6) )


###################################################
### code chunk number 13: BrCaMS.rnw:255-257
###################################################
xx.rd <- update( x.rd, . ~ . + Ns( tfd-tfr, knots=ka.rd) )
anova( m.rd, x.rd, xx.rd, test="Chisq" )


###################################################
### code chunk number 14: rd-int
###################################################
nd <- data.frame( expand.grid( tfd=c(NA,seq(0,15,0.1)), 
                               tad=c(0,0.5,1,2,3,5,8) ), 
                  lex.dur=100 )
nd <- subset( transform( nd, tfr = tfd - tad ), tfr>=0 | is.na(tfr) )
head( nd )
matplot( nd$tfd, cbind( ci.pred( x.rd, nd )[,1],
                        ci.pred(xx.rd, nd )[,1] ),
         type="l", lty=c("solid","22"), lend="butt", 
         lwd=3, col=clr[3], las=1,
         log="y", xlab="Time since diagnosis (years)",
                  ylab="Mortality rate per 100 PY" )
matlines( seq(0,15,0.1), rd.rate,
          type="l", lwd=c(3,1,1), lty=1, col=gray(0.3) )


###################################################
### code chunk number 15: BrCaMS.rnw:313-317
###################################################
c.ar <- update( m.ar, . ~ . + age + size + nodes + pr.tr + hormon )
c.ad <- update( m.ad, . ~ . + age + size + nodes + pr.tr + hormon )
c.rd <- update( m.rd, . ~ . + age + size + nodes + pr.tr + hormon )
cx.rd<- update(xx.rd, . ~ . + age + size + nodes + pr.tr + hormon )


###################################################
### code chunk number 16: int-test (eval = FALSE)
###################################################
## int.test <- NArray( list( model=c("c.ar","c.ad","c.rd","cx.rd"),
##                             var=c("age","size","nodes","pr.tr","hormon"),
##                            what=c("d.f.","Dev","P") ) )
## str( int.test )
## 
## int.test[1,1,]<-as.numeric(anova( c.ar,update( c.ar,.~.+log(tfd+0.5):age   ),test="Chisq")[2,3:5])
## int.test[1,2,]<-as.numeric(anova( c.ar,update( c.ar,.~.+log(tfd+0.5):size  ),test="Chisq")[2,3:5])
## int.test[1,3,]<-as.numeric(anova( c.ar,update( c.ar,.~.+log(tfd+0.5):nodes ),test="Chisq")[2,3:5])
## int.test[1,4,]<-as.numeric(anova( c.ar,update( c.ar,.~.+log(tfd+0.5):pr.tr ),test="Chisq")[2,3:5])
## int.test[1,5,]<-as.numeric(anova( c.ar,update( c.ar,.~.+log(tfd+0.5):hormon),test="Chisq")[2,3:5])
## 
## int.test[2,1,]<-as.numeric(anova( c.ad,update( c.ad,.~.+log(tfd+0.5):age   ),test="Chisq")[2,3:5])
## int.test[2,2,]<-as.numeric(anova( c.ad,update( c.ad,.~.+log(tfd+0.5):size  ),test="Chisq")[2,3:5])
## int.test[2,3,]<-as.numeric(anova( c.ad,update( c.ad,.~.+log(tfd+0.5):nodes ),test="Chisq")[2,3:5])
## int.test[2,4,]<-as.numeric(anova( c.ad,update( c.ad,.~.+log(tfd+0.5):pr.tr ),test="Chisq")[2,3:5])
## int.test[2,5,]<-as.numeric(anova( c.ad,update( c.ad,.~.+log(tfd+0.5):hormon),test="Chisq")[2,3:5])
## 
## int.test[3,1,]<-as.numeric(anova( c.rd,update( c.rd,.~.+log(tfd+0.5):age   ),test="Chisq")[2,3:5])
## int.test[3,2,]<-as.numeric(anova( c.rd,update( c.rd,.~.+log(tfd+0.5):size  ),test="Chisq")[2,3:5])
## int.test[3,3,]<-as.numeric(anova( c.rd,update( c.rd,.~.+log(tfd+0.5):nodes ),test="Chisq")[2,3:5])
## int.test[3,4,]<-as.numeric(anova( c.rd,update( c.rd,.~.+log(tfd+0.5):pr.tr ),test="Chisq")[2,3:5])
## int.test[3,5,]<-as.numeric(anova( c.rd,update( c.rd,.~.+log(tfd+0.5):hormon),test="Chisq")[2,3:5])
## 
## int.test[4,1,]<-as.numeric(anova(cx.rd,update(cx.rd,.~.+log(tfd+0.5):age   ),test="Chisq")[2,3:5])
## int.test[4,2,]<-as.numeric(anova(cx.rd,update(cx.rd,.~.+log(tfd+0.5):size  ),test="Chisq")[2,3:5])
## int.test[4,3,]<-as.numeric(anova(cx.rd,update(cx.rd,.~.+log(tfd+0.5):nodes ),test="Chisq")[2,3:5])
## int.test[4,4,]<-as.numeric(anova(cx.rd,update(cx.rd,.~.+log(tfd+0.5):pr.tr ),test="Chisq")[2,3:5])
## int.test[4,5,]<-as.numeric(anova(cx.rd,update(cx.rd,.~.+log(tfd+0.5):hormon),test="Chisq")[2,3:5])
## save( int.test, file="int-test.Rda")


###################################################
### code chunk number 17: BrCaMS.rnw:363-367
###################################################
load( file="int-test.Rda")
round( int.test[,,2], 2 )
round( int.test[,,3], 4 )
round( int.test[,,1], 0 )


###################################################
### code chunk number 18: BrCaMS.rnw:392-402
###################################################
 i.ar <- update(  c.ar, . ~ . + log(tfd+0.5):size 
                              + log(tfd+0.5):pr.tr
                              + log(tfd+0.5):hormon )
 i.ad <- c.ad
 i.rd <- update(  c.rd, . ~ . + log(tfd+0.5):pr.tr )
ix.rd <- update( xx.rd, . ~ . + log(tfd+0.5):pr.tr )
round( ci.lin( i.ad ), 4 )
round( ci.lin( i.ar ), 4 )
round( ci.lin( i.rd ), 4 )
round( ci.lin( cx.rd ), 4 )


###################################################
### code chunk number 19: BrCaMS.rnw:416-437
###################################################
nd.size <- data.frame( tfd = rep( c(NA,seq(0,15,0.1)), 3 ),
              lex.dur = 100,
                  age = 45,
                 size = rep( levels(Lbc$size), each=152 ),
                nodes = 5,
                pr.tr = 3,
               hormon = levels(Lbc$hormon)[1] )
nd.pr <- data.frame( tfd = rep( c(NA,seq(0,15,0.1)), 6 ),
              lex.dur = 100,
                  age = 45,
                 size = levels(Lbc$size)[2],
                nodes = 5,
                pr.tr = rep( 0:5, each=152 ),
               hormon = levels(Lbc$hormon)[1] )
nd.hormon <- data.frame( tfd = rep( c(NA,seq(0,15,0.1)), 2 ),
              lex.dur = 100,
                  age = 45,
                 size = levels(Lbc$size)[2],
                nodes = 5,
                pr.tr = 3,
               hormon = rep( levels(Lbc$hormon), each=152 ) )


###################################################
### code chunk number 20: int-size
###################################################
clr <- rainbow(3) ; yl <- c(0.03,60)
ad.c.rate <- ci.pred( c.ad, nd.size ) ; ad.i.rate <- ci.pred( i.ad, nd.size )
ar.c.rate <- ci.pred( c.ar, nd.size ) ; ar.i.rate <- ci.pred( i.ar, nd.size )
rd.c.rate <- ci.pred( c.rd, nd.size ) ; rd.i.rate <- ci.pred( i.rd, nd.size )
matplot( nd.size$tfd, cbind( ad.c.rate, ad.i.rate,
                             ar.c.rate, ar.i.rate,
                             rd.c.rate, rd.i.rate ),
         type="l", lty=rep(c("22","solid"),each=3), 
                   lwd=c(2,0,0), 
                   col=rep(clr,each=6), las=1, lend="butt",
         log="y", xlab="Time since diagnosis (years)",
         ylim=yl, ylab="Rate per 100 PY" )
text( par("usr")[2]*0.95, (10^par("usr"))[3]*1.4^(1:3), 
      c("A->D","A->R","R->D"), col=clr, adj=1, font=2 )


###################################################
### code chunk number 21: int-pr
###################################################
ad.c.rate <- ci.pred( c.ad, nd.pr ) ; ad.i.rate <- ci.pred( i.ad, nd.pr )
ar.c.rate <- ci.pred( c.ar, nd.pr ) ; ar.i.rate <- ci.pred( i.ar, nd.pr )
rd.c.rate <- ci.pred( c.rd, nd.pr ) ; rd.i.rate <- ci.pred( i.rd, nd.pr )
matplot( nd.pr$tfd, cbind( ad.c.rate, ad.i.rate,
                           ar.c.rate, ar.i.rate,
                           rd.c.rate, rd.i.rate ),
         type="l", lty=rep(c("22","solid"),each=3), 
                   lwd=c(2,0,0), 
                   col=rep(clr,each=6), las=1, lend="butt",
         log="y", xlab="Time since diagnosis (years)",
         ylim=yl, ylab="Rate per 100 PY" )
text( par("usr")[2]*0.95, (10^par("usr"))[3]*1.4^(1:3), 
      c("A->D","A->R","R->D"), col=clr, adj=1, font=2 )


###################################################
### code chunk number 22: int-hormon
###################################################
ad.c.rate <- ci.pred( c.ad, nd.hormon ) ; ad.i.rate <- ci.pred( i.ad, nd.hormon )
ar.c.rate <- ci.pred( c.ar, nd.hormon ) ; ar.i.rate <- ci.pred( i.ar, nd.hormon )
rd.c.rate <- ci.pred( c.rd, nd.hormon ) ; rd.i.rate <- ci.pred( i.rd, nd.hormon )
matplot( nd.hormon$tfd, cbind( ad.c.rate, ad.i.rate,
                               ar.c.rate, ar.i.rate,
                               rd.c.rate, rd.i.rate ),
         type="l", lty=rep(c("22","solid"),each=3), 
                   lwd=c(2,0,0), 
                   col=rep(clr,each=6), las=1, lend="butt",
         log="y", xlab="Time since diagnosis (years)",
         ylim=yl, ylab="Rate per 100 PY" )
text( par("usr")[2]*0.95, (10^par("usr"))[3]*1.4^(1:3), 
      c("A->D","A->R","R->D"), col=clr, adj=1, font=2 )


###################################################
### code chunk number 23: BrCaMS.rnw:552-568
###################################################
names( Rbc )
Lini <- Rbc[NULL,c("tfd","A","P","tfr",
                   "lex.Cst","lex.Xst","lex.dur","lex.id",
                   "age","size","nodes","pr.tr","hormon")]
pr.nodes <- seq(0,20,5)
npr <- nlevels(Rbc$size) * length(pr.nodes)
Lini[1:npr,"tfd"] <- 0
Lini[1:npr,"tfr"] <- NA
Lini[1:npr,"lex.Cst"] <- "Alive"
Lini[1:npr,"age"] <- 54
Lini[1:npr,"size"] <- rep( levels(Rbc$size), length(pr.nodes) )
Lini[1:npr,"nodes"] <- rep( pr.nodes, each=nlevels(Rbc$size) )
Lini[1:npr,"pr.tr"] <- 3
Lini[1:npr,"hormon"] <- "no"
Lini
str( Lini )


###################################################
### code chunk number 24: BrCaMS.rnw:580-588
###################################################
TR  <- list( Alive = list( Dead = i.ad,
                            Rel = i.ar ),
               Rel = list( "Dead(Rel)" = i.rd ) )
TRx <- list( Alive = list( Dead = i.ad,
                            Rel = i.ar ), 
               Rel = list( "Dead(Rel)" = ix.rd ) )
lapply( TR, names )
lapply( TR, lapply, class )


###################################################
### code chunk number 25: simL (eval = FALSE)
###################################################
## sL  <- simLexis( Tr=TR , init=Lini, N=2000, t.range=16 )
## sLx <- simLexis( Tr=TRx, init=Lini, N=2000, t.range=16 )
## save( sL, sLx, file="sL.Rda" )


###################################################
### code chunk number 26: loadsim
###################################################
load( file="sL.Rda" )
summary( sLx )


###################################################
### code chunk number 27: nStstr
###################################################
nn <- nState( sLx[1:1000,], at=seq(0,16,0.1), from=0, time.scale="tfd" )
pp <- pState( nn, perm=c(1,2,4,3) )
str( pp )


###################################################
### code chunk number 28: defNArr
###################################################
( tt <- with( sLx, table( nodes, size ) ) )
prX <- prA <- NArray( c( dimnames( tt ), dimnames( pp ) ) )
str( prA )


###################################################
### code chunk number 29: fillNArr (eval = FALSE)
###################################################
## for( nn in dimnames(prA)[[1]] )
## for( ss in dimnames(prA)[[2]] )
##    {
## prA[nn,ss,,] <- pState( nState( subset( sL , nodes==as.numeric(nn) &
##                                               size==ss ),
##                                 at = seq(0,16,0.1), 
##                               from = 0, 
##                         time.scale = "tfd" ),
##                         perm = c(1,2,4,3) )
## prX[nn,ss,,] <- pState( nState( subset( sLx, nodes==as.numeric(nn) &
##                                               size==ss ),
##                                 at = seq(0,16,0.1), 
##                               from = 0, 
##                         time.scale = "tfd" ),
##                         perm = c(1,2,4,3) )
##    }
## save( prA, prX, file="pr.Rda" )


###################################################
### code chunk number 30: states
###################################################
load( file="pr.Rda" )
clr <- col2rgb( c("forestgreen","maroon") )
clr <- cbind( clr, clr[,2:1]*0.6 + matrix(255,3,2)*0.4 )
clr <- rgb( t(clr), max=255 ) 
par( mfrow=c(3,3), mar=c(1,1.5,1,1), mgp=c(3,1,0)/1.6, oma=c(2,2,2,2) )
nnn <- dimnames(prA)[[1]]
sss <- dimnames(prA)[[2]]
for( nn in nnn[c(1,3,5)] ) # only nodes as in the SiM paper
for( ss in sss )
   {
   plot.pState( prX[nn,ss,,], col=clr, xlim=c(0,15), ylab="", xlab="" )
   lines( as.numeric(dimnames(prX)[[3]]), prX[nn,ss,,  2], lwd=3, lty=1, col="black" ) 
matlines( as.numeric(dimnames(prA)[[3]]), prA[nn,ss,,1:3], lwd=1, lty=1, col="white" ) 
   axis( side=2, at=0:10/10, labels=NA, tcl=-0.4 )
   axis( side=4, at=0:10/10, labels=NA, tcl=-0.4 )
   axis( side=2, at=0:50/50, labels=NA, tcl=-0.2 )
   axis( side=4, at=0:50/50, labels=NA, tcl=-0.2 )
   }
mtext( paste( "Size"  ,sss), side=3, at=c(1,3,5)/6, outer=TRUE, line=0, cex=0.66, las=0 )
mtext( paste( "Nodes=",nnn[c(1,3,5)]), side=4, at=c(5,3,1)/6, outer=TRUE, line=0, cex=0.66, las=0 )
mtext( "Time since diagnosis (years)", side=1,      outer=TRUE, line=1, cex=0.66, las=0 )
mtext( "Probability", side=2, outer=TRUE, line=1, cex=0.66, las=0 )


###################################################
### code chunk number 31: BrCaMS.rnw:692-705
###################################################
cA <- apply( prA[,,1:151,1:3], c(1,2,4),
             function(x) (sum(x[-1])+sum(x[-length(x)]))/2 * 1/10 )
cA[,,3] <- cA[,,2] - cA[,,1]
dimnames( cA )[[3]] <- c("noRel","Total","Rel")
cA <- cA[,,c(1,3,2)]
round( ftable( cA, row.vars=c(3,2) ), 2 )

cX <- apply( prX[,,1:151,1:3], c(1,2,4),
             function(x) (sum(x[-1])+sum(x[-length(x)]))/20 )
cX[,,3] <- cX[,,2] - cX[,,1]
dimnames( cX )[[3]] <- c("noRel","Total","Rel")
cX <- cX[,,c(1,3,2)]
round( ftable( cX, row.vars=c(3,2) ), 2 )


###################################################
### code chunk number 32: BrCaMS.rnw:742-753
###################################################
mbc <- mcutLexis( Lbc,
            timescale = "tfd", 
                   wh = c("tor","tom"),
     precursor.states = "Alive",
           new.states = c("Rel","Met"),
           seq.states = TRUE,
           new.scales = c("tfr","tfm") )
summary( mbc, timeScale = TRUE )
mbc <- Relevel( mbc, list( 1, 3, Met=4:5, 2 ) )
summary( mbc )
subset( mbc, lex.id %in% (1328+0:2) )[,1:10]


###################################################
### code chunk number 33: BrCaMS.rnw:757-766
###################################################
xbc <- transform( mbc, lex.Xst = factor( ifelse( lex.Xst=="Dead" & 
                                                 lex.Cst!="Alive",
                                                 paste( "D(",lex.Cst,")",sep=""),
                                                 as.character(lex.Xst) ) ) )
xbc <- Relevel( xbc )
levels( xbc )
xbc <- Relevel( xbc, c(1:3,5,4) )
levels( xbc )
summary( xbc )


###################################################
### code chunk number 34: box-rmx
###################################################
boxes( xbc, boxpos=list(x=c(15,40,15,85,85,85),
                        y=c(85,50,15,15,85,50)),
            show.BE=TRUE, scale.R=100, wmult=1.1 )


###################################################
### code chunk number 35: BrCaMS.rnw:844-845 (eval = FALSE)
###################################################
## Tr <- lapply( Tr, lapply, get )


###################################################
### code chunk number 36: BrCaMS.rnw:852-853 (eval = FALSE)
###################################################
## bootTr <- lapply( Tr, lapply, function(x) paste("BOOT",x,sep="") )


###################################################
### code chunk number 37: BrCaMS.rnw:857-863 (eval = FALSE)
###################################################
## unique.models <- unique( unlist( Tr ) )
## for( m in unique.models )
##    {
##    assign( paste("BOOT",m,sep=""),
##            update( get(m), data=boot.Lexis(data) ) )
##    }


